PlotPoints <- function(X, labels, colors, title = ""){
  library(deldir)
  voronoi <- deldir(X[,1], X[,2])
  plot(X[,1], X[,2], type="n", asp=1, xlab = "", ylab = "", main = title)
  points(X[,1], X[,2], pch=20, col=colors[,1], cex=1.3)
  text(X[,1], X[,2], labels = labels[,1], pos = 1, cex = 0.6)
  plot(voronoi, wlines="tess", wpoints="none", number=FALSE, add=TRUE, lty=1)
  legend("topleft", legend = sort(unique(colors[,1])), col=sort(unique(colors[,1])), pch=20, cex = 0.8)
}

Classify <- function(X, colors, kfolds = 50){
  #install.packages('tree')
  library(tree)
  #library(caret)
  set.seed(17)
  #add class
  if(!any(names(X) == "class")){X <- cbind(X, class=as.factor(colors))}
  #randomly shuffle the data
  data <-X[sample(nrow(X)),]
  #Create 10 equally size folds
  folds <- cut(seq(1,nrow(data)),breaks=kfolds,labels=FALSE)
  acc <- rep(0, times = kfolds)
  #10 fold cross validation
  for(i in 1:kfolds){
    #Segment your data by fold using the which() function 
    testIndexes <- which(folds==i,arr.ind=TRUE)
    testData <- data[testIndexes, ]
    trainData <- data[-testIndexes, ]
    model <- tree(formula = class ~., data = trainData)
    pre <- predict(model, newdata=testData, y = TRUE, type="class")
    acc[i] <- sum(pre == testData$class)/length(testData$class)
  }
  return(acc)
}

############################### MAIN #############################################

#Load the dataset of 165 words
mydata <- read.csv('data.csv', header = FALSE)
mylabels <- read.csv('labels.txt', header = FALSE)
mycolors <- read.csv('colors.csv', header = FALSE)

PlotPoints(mydata[,c(1,2)], mylabels, mycolors)

#ADD YOUR CODE HERE!
#1.Determine the neighbors of each point (k-NN) [0.5p]

#2.Construct a neighborhood graph. [1p]
#Each point is connected to other if it is a K nearest neighbor.
#Edge length equal to Euclidean distance.

#3.Compute shortest path between two nodes. (Floyd–Warshall algorithm) [0.5p]

#4.(classical) multidimensional scaling
fit <- cmdscale(D,eig=TRUE, k=2)
PlotPoints(fit$points, mylabels, mycolors, title = "ISOMAP")

#Principal component analysis
fitPCA <- prcomp(mydata, center = TRUE, scale. = TRUE)
PlotPoints(fitPCA$x[,c(1,2)], mylabels, mycolors, title = "PCA")

#t-SNE (T-Distributed Stochastic Neighbor Embedding)
#install.packages('tsne')
library(tsne)
fittsne <- tsne(mydata, k = 2)
PlotPoints(fittsne, mylabels, mycolors, title = "tSNE")

#classify ISOMAP
accISOMAP <- Classify(as.data.frame(fit$points), mycolors$V1, 50)

#classify PCA
accPCA <- Classify(as.data.frame(fitPCA$x), mycolors$V1, 50)

#classify t-SNE
accTSNE <- Classify(as.data.frame(fittsne), mycolors$V1, 50)

#PLOT results
write(paste("ISOMAP ACC:", mean(accISOMAP)), stdout())
write(paste("PCA ACC:", mean(accPCA)), stdout())
write(paste("t-SNE ACC:", mean(accTSNE)), stdout())
